package ga4robot;

import java.util.ArrayList;
import java.util.Random;

/**
 * 染色体
 * @author 邢兴
 */
public class Chromosome {
    /**
     * 基因组
     */
    private ArrayList<Gene> genes = null;
    /**
     * 染色体构造函数
     * @param n     基因位数目
     */
    public Chromosome(int n) {
        genes = new ArrayList<Gene>(n);
    }

    private Chromosome() {

    }

    /**
     * 在 id 位上设置基因 gene
     * @param id    基因位
     * @param gene  基因
     */
    public void setGene(int id, Gene gene) {
        genes.set(id, gene);
    }

    /**
     * 在 id 位上添加基因 gene
     * @param id    基因位
     * @param gene  基因
     */
    public void addGene(int id, Gene gene) {
        genes.add(id, gene);
    }
    /**
     * 获取 id 位的基因
     * @param id    基因位
     * @return      基因
     */
    public Gene getGene(int id) {
        return genes.get(id);
    }

    /**
     * 重载 clone 方法
     * @return      染色体
     */
    @Override
    public Chromosome clone() {
        Chromosome chromosome = new Chromosome();
        chromosome.genes = (ArrayList<Gene>)genes.clone();
        return chromosome;
    }
    /**
     * 交叉
     * @param chromosome    与当前染色体交叉的令一染色体
     * @return              交叉得到新的两条染色体
     */
    public ArrayList<Chromosome> crossover(Chromosome chromosome) {
        Random r = new Random();
        int x = r.nextInt(genes.size());
        ArrayList<Chromosome> list = new ArrayList<Chromosome>(2);                
        Chromosome c1 = new Chromosome(genes.size());
        Chromosome c2 = new Chromosome(genes.size());
        for(int i = 0; i < genes.size(); i ++) {
            if (i < x) {
                c1.addGene(i, genes.get(i));
                c2.addGene(i, chromosome.getGene(i));
            } else {               
                c1.addGene(i, chromosome.getGene(i));
                c2.addGene(i, genes.get(i));                
            }
        }
        list.add(c1);
        list.add(c2);
        return list;
    }
    /**
     * 变异
     */
    public void mutation() {
        Random gr = new Random();        
        // 选取随机一个位置的基因
        int id = gr.nextInt(genes.size());
        GeneGroup group = GeneGroup.getGroup(id);
        //从基因组中随意抽取一个，替换所选基因
        genes.set(id, group.getRandomGene());
    }
    /**
     * 评估函数
     * @return 评估函数值
     */
    public double evaluation() {
        double value = 0;
        for(int i = 0; i < genes.size(); i ++) {
            value += genes.get(i).getValue();
        }
        // 确保评估函数 eval(x) > 0
        value += genes.size() * 30;
        return value;
    }
}
