package ga4robot;

import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Properties;

/**
 * 配置文件
 * @author 邢兴
 */
public class Config {
    /**
     * 种群大小
     */
    private int population = 0;
    /**
     * 基因位数
     */
    private int geneCount = 0;
    /**
     * 交叉概率
     */
    private double crossoverPossible = 0;
    /**
     * 变异概率
     */
    private double mutationPossible = 0;
    /**
     * 子代数
     */
    private int generationCount = 0;
    /**
     * 最优解稳定次数
     */
    private int maxEqualsCount = 0;
    /**
     * 装备最大数量
     */
    private int maxGenes = 0;

    /**
     * 配置构造函数
     * @throws java.io.IOException
     */
    public Config() throws IOException {
        Properties p = new Properties();
        p.load(new FileInputStream("config.property"));
        generationCount = Integer.parseInt(p.getProperty("generationCount"));
        maxEqualsCount = Integer.parseInt(p.getProperty("maxEqualsCount"));
        population = Integer.parseInt(p.getProperty("population"));
        geneCount = Integer.parseInt(p.getProperty("geneCount"));
        crossoverPossible = Double.parseDouble(p.getProperty("crossoverPossible"));
        mutationPossible = Double.parseDouble(p.getProperty("mutationPossible"));
        maxGenes = Integer.parseInt(p.getProperty("maxGenes"));
    }
    
    /**
     * 初始化
     * @throws java.io.IOException
     */
    public static void init() throws IOException {
        Properties p = new Properties();
        p.setProperty("generationCount", "100000");
        p.setProperty("maxEqualsCount", "500");
        p.setProperty("population", "100");
        p.setProperty("geneCount", "4");
        p.setProperty("crossoverPossible", "0.8");
        p.setProperty("mutationPossible", "0.1");
        p.setProperty("maxGenes", "50");
        p.store(new FileOutputStream("config.property"), "for GA");
    }

    /**
     *
     * @return 子代数量
     */
    public int getGenerationCount() {
        return generationCount;
    }

    /**
     *
     * @return 可能最优解稳定次数
     */
    public int getMaxEqualsCount() {
        return maxEqualsCount;
    }
    /**
     *
     * @return 种群数量
     */
    public int getPopulation() {
        return population;                
    }

    /**
     *
     * @return 基因位数
     */
    public int getGeneCount() {
        return geneCount;
    }

    /**
     *
     * @return 交叉概率
     */
    public double getCrossoverPossible() {
        return crossoverPossible;
    }

    /**
     *
     * @return 变异概率
     */
    public double getMutationPossible() {
        return mutationPossible;
    }

    /**
     * 
     * @return 每基因位可选最大基因数
     */
    public int getMaxGenes() {
        return maxGenes;
    }

    /**
     * 单件模式
     */
    private static Config config = null;

    public static Config singlation() throws IOException {
        if (config == null) {
            config = new Config();
        }
        return config;
    }
}
