package ga4robot;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.Random;
import javax.xml.parsers.*;
import org.w3c.dom.Document;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;


/**
 * 代选基因组
 * @author 邢兴
 */
public class GeneGroup {

    /**
     * 基因组
     */
    private ArrayList<Gene> genes = null;
    /**
     * 基因位
     */
    private int id = 0;

    /**
     * 基因组构造函数
     * @param id    基因位
     * @param n     基因组大小
     */
    public GeneGroup(int id, int n) {
        genes = new ArrayList<Gene>(n);        
        this.id = id;
    }

    /**
     * 设置基因
     * @param id    基因位
     * @param gene  基因
     */
    public void setGene(int id, Gene gene) {
        genes.set(id, gene);
    }
    /**
     * 添加基因
     * @param id    基因位
     * @param gene  基因
     */
    public void addGene(int id, Gene gene) {
        genes.add(id, gene);
    }

    /**
     * 获取基因
     * @param id    基因位
     * @return      基因
     */
    public Gene getGene(int id) {
        return genes.get(id);
    }       

    /**
     * 随机获取基因
     * @return  基因
     */
    public Gene getRandomGene() {
        Random r = new Random();        
        return genes.get(r.nextInt(genes.size()));
    }

    /**
     * 基因组大小
     * @return
     */
    public int getSize() {
        return genes.size();
    }
    
    /**
     * 单件模式
     */
    private static ArrayList<GeneGroup> group = null;

    /**
     * 随机初始化测试数据
     * @param n 基因位数
     * @throws java.io.IOException
     */
    public static void initGroup(int n) throws IOException {
        group = new ArrayList<GeneGroup>(n);
        for(int ggi = 0; ggi < n; ggi ++) {
            Random ggr = new Random();
            int ggmax = ggr.nextInt(Config.singlation().getMaxGenes()) + 1;
            // 每个基因组有 1 .. MAX_GENE_IN_GROUP 个待选基因
            GeneGroup g = new GeneGroup(ggi, ggmax);
            for(int gi = 0; gi < ggmax; gi ++) {
                Random gr = new Random();
                // 每个基因价值在 -30 到 70 之间
                double value = (gr.nextDouble() - 0.3) * 100;                
                Gene gene = new Gene(gi, value);
                g.addGene(gi, gene);
            }
            group.add(ggi, g);
        }
    }   

    /**
     * 保存基因组
     * @param config    输出流
     * @throws java.io.IOException
     * @throws org.xml.sax.SAXException
     * @throws javax.xml.parsers.ParserConfigurationException
     */
    public static void saveGroup(OutputStream config) throws IOException, SAXException, ParserConfigurationException { 
        config.write("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n<root>\n".getBytes());
        for(GeneGroup gg: group) {                        
            config.write("\t<genegroup>\n".getBytes());
            for(Gene g : gg.genes) {                
                config.write(("\t\t<gene>" + g.getValue() + "</gene>\n").getBytes());
            }            
            config.write("\t</genegroup>\n".getBytes());
        }        
        config.write("</root>\n".getBytes());        
    }

    /**
     * 加载基因组
     * @param config    输入流
     * @throws java.io.IOException
     * @throws org.xml.sax.SAXException
     * @throws javax.xml.parsers.ParserConfigurationException
     */
    public static void loadGroup(InputStream config) throws IOException, SAXException, ParserConfigurationException {        
        Document doc = DocumentBuilderFactory.newInstance().
                newDocumentBuilder().parse(config);        
        NodeList ggNode = doc.getElementsByTagName("genegroup");
        group = new ArrayList<GeneGroup>(ggNode.getLength());
        for(int ggi = 0; ggi < ggNode.getLength(); ggi ++) {
            NodeList gNode = ggNode.item(ggi).getChildNodes();
            int gmax = gNode.getLength();
            GeneGroup g = new GeneGroup(ggi, gmax);
            int i = 0;
            for(int gi = 0; gi < gmax; gi ++) {                             
                if (gNode.item(gi).getNodeName().equals("gene")) {
                    double value = Double.parseDouble(gNode.item(gi).getTextContent());
                    Gene gene = new Gene(gi, value);
                    g.addGene(i, gene);
                    i ++;
                }
            }
            group.add(ggi, g);
        }        
    }

    /**
     * 获取组
     * @param id    基因位
     * @return  基因组
     */
    public static GeneGroup getGroup(int id) {
        return group.get(id);    
    }   

    /**
     *
     * @return 基因组数，即基因位数
     */
    public static int getGroupSize() {
        return group.size();
    }
}
