package ga4robot;

import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.util.Date;
import javax.xml.parsers.ParserConfigurationException;
import org.xml.sax.SAXException;

/**
 * 主程序
 * @author 邢兴
 */
public class Main {

    /**
     * 开始时间
     */
    private static long startTime = 0;

    /**
     * 记录开始时间
     */
    private static void startTime() {
        Date d = new Date();
        startTime = d.getTime();
    }

    /**
     *
     * @return 返回运行时间
     */
    private static long endTime() {
        Date d = new Date();
        return d.getTime() - startTime;
    }

    /**
     * 生成配置文件
     * @throws java.io.FileNotFoundException
     * @throws java.io.IOException
     * @throws org.xml.sax.SAXException
     * @throws javax.xml.parsers.ParserConfigurationException
     */
    private static void makeConfig() throws FileNotFoundException, IOException,
            SAXException, ParserConfigurationException {
        GeneGroup.initGroup(Config.singlation().getGeneCount());
        OutputStream os = new FileOutputStream("config.xml");
        GeneGroup.saveGroup(os);
        os.close();
    }

    /**
     * 穷举
     * @return 最优解评估函数值
     * @throws java.io.FileNotFoundException
     * @throws java.io.IOException
     * @throws org.xml.sax.SAXException
     * @throws javax.xml.parsers.ParserConfigurationException
     */
    private static double exhaustingCount() throws FileNotFoundException,
            IOException, SAXException, ParserConfigurationException {
        InputStream config = new FileInputStream("config.xml");
        GeneGroup.loadGroup(config);

        // 遍历键值
        int key[] = new int[GeneGroup.getGroupSize()];        
        // 总数
        int total = 1;
        for (int i = 0; i < GeneGroup.getGroupSize(); i++) {
            key[i] = 0;            
            total *= GeneGroup.getGroup(i).getSize();
        }
        System.out.println("Total: " + total);
        double result = 0;
       
        for (int i = 0; i < total; i++) {
            Chromosome c = new Chromosome(GeneGroup.getGroupSize());
             for (int k = 0; k < GeneGroup.getGroupSize(); k++) {
                 c.addGene(k, GeneGroup.getGroup(k).getGene(key[k]));
            }
            if (result < c.evaluation()) {
                result = c.evaluation();
            }

            for(int j = 1; j <= GeneGroup.getGroupSize(); j ++) {
                key[j - 1] ++;
                if (key[j - 1] == GeneGroup.getGroup(j - 1).getSize()) {
                    key[j - 1] = 0;                    
                } else {
                    break;
                }
            }
        }

        return result;
    }

    /**
     * 遗传算法
     * @return 最优解评估函数值
     * @throws java.io.FileNotFoundException
     * @throws java.io.IOException
     * @throws org.xml.sax.SAXException
     * @throws javax.xml.parsers.ParserConfigurationException
     */
    private static double runGA() throws FileNotFoundException, IOException,
            SAXException, ParserConfigurationException {
        //begin-初始化
        InputStream config = new FileInputStream("config.xml");
        Generation g = new Generation(config);
        double best = 0, newbest = 0;
        Chromosome gbest = null;
        int i = 0, j = 0;
        //end-初始化
        startTime();
        while ((i < Config.singlation().getGenerationCount()) &&
                (j < Config.singlation().getMaxEqualsCount())) {
            g.newGeneration();
            gbest = g.getBest();
            best = gbest.evaluation();
            if (best != newbest) {
                newbest = best;
                j = 0;
            } else {
                j++;
            }
            //System.out.println(best);
            i++;
        }
        System.out.println("Last best: " + best);
        System.out.println("History best: " + g.getHistoryBest().evaluation());
        System.out.println("Generation Number: " + i);
        System.out.println("Count of Equals: " + j);
        System.out.println("Time usage: " + endTime() + " ms");

        return best > g.getHistoryBest().evaluation() ? best : g.getHistoryBest().evaluation();
    }

    /**
     * 10 次 GA
     * @throws java.io.FileNotFoundException
     * @throws java.io.IOException
     * @throws org.xml.sax.SAXException
     * @throws javax.xml.parsers.ParserConfigurationException
     */
    public static void variance() throws FileNotFoundException, IOException,
            SAXException, ParserConfigurationException {
        double ga[] = new double[10];
        double total = 0;
        for (int i = 0; i < 10; i++) {
            ga[i] = runGA();
            total += ga[i];
        }
        double average = total / 10;
        double result = 0;
        for (int i = 0; i < 10; i++) {
            result += Math.pow(Math.abs(ga[i] - average), 2);
            System.out.println(i + ": " + ga[i]);
        }
        System.out.println("Average:" + average);
        System.out.println("Variance:" + result / 9);
    }

    /**
     * @param args the command line arguments
     */
    public static void main(String[] args) {
        try {
            System.out.println("Select something to do:");
            System.out.println("\t1. Making config.xml");
            System.out.println("\t2. Exhausting Count");
            System.out.println("\t3. Computing GA");
            System.out.println("\t4. Variance");
            System.out.print("[1-4]:");
            BufferedReader br = new BufferedReader(new InputStreamReader(System.in));
            switch (Integer.valueOf(br.readLine())) {
                case 1:
                    makeConfig();
                    break;
                case 2:
                    startTime();
                    System.out.println(exhaustingCount());
                    System.out.println("Time usage: " + endTime() + " ms");
                    break;
                case 3:
                    runGA();
                    break;
                case 4:
                    variance();
                    break;
                case 99:
                    Config.init();
                    break;
                default:
                    System.out.println("Only 1 to 4 accepted!");
                    break;
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
